-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     fetch.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-10-29
-------------------------------------------------------------------------------
-- Brief    Instruction Fetch pipeline stage. Interfaces the Instruction Memory. 
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;


library work;
use work.riscv_pkg.all;

entity fetch is
  generic (RESET_VECTOR : std_logic_vector(XLEN-1 downto 0) := (others => '0'));
  port (
    -- Entrées externes au pipeline
    i_clk :       in std_logic;
    i_rstn :      in std_logic;

    -- Entrées provenant de l'étage Execute
    i_stall :     in std_logic;
    i_transfert : in std_logic;
    i_target :    in std_logic_vector(XLEN-1 downto 0);
    i_flush :     in std_logic;

    -- Interface vers la mémoire d'instruction
    i_mem_read :  in std_logic_vector(XLEN-1 downto 0);
    o_mem_addr :  out std_logic_vector(IMEM_ADDR_WIDTH-1 downto 0);
    
    -- Sorties
    o_fetch_reg : out fetch_reg);
end entity fetch;

architecture beh of fetch is

  signal pc         : std_logic_vector(XLEN-1 downto 0);
  signal fetch_data : std_logic_vector(XLEN-1 downto 0);

begin

pc_inst: entity work.riscv_pc
  generic map (
    RESET_VECTOR => RESET_VECTOR
  )
  port map (
    i_clk       => i_clk,
    i_rstn      => i_rstn,
    i_stall     => i_stall,
    i_transfert => i_transfert,
    i_target    => i_target,
    o_pc        => pc
  );

o_mem_addr <= pc(IMEM_ADDR_WIDTH-1 downto 0);

-- Gérer les flush et les stall
flush_or_stall_pipeline:
process (all) is
begin
  if i_flush = '1' then
    fetch_data <= (others => '0');
  elsif i_stall = '1' then
    fetch_data <= o_fetch_reg.instruction;
  else
    fetch_data <= i_mem_read;
  end if;
end process flush_or_stall_pipeline;

-- Écrire le registre IF/ID
write_fetch_reg:
process (i_clk) is
begin
  if rising_edge(i_clk) then
    if i_rstn = '1' then
      o_fetch_reg.instruction <= fetch_data;--i_mem_read;
      o_fetch_reg.pc <= pc;
    else
      o_fetch_reg.instruction <= (others => '0');
      o_fetch_reg.pc <= (others => '0');
    end if;
  end if;
end process write_fetch_reg;

-- FIXME: Reset pas pris en compte
--fetching: 
--process (i_clk, i_rstn) is
--  begin
--    if rising_edge(i_clk) and i_flush='1' then
--      fetch <= std_logic_vector(RESET_VECTOR);
--
--    elsif rising_edge(i_clk) and i_flush='0' then
--      fetch <= i_mem_read;
--
--    elsif rising_edge(i_clk) and i_stall = '1' then
--      o_fetch <= o_fetch;
--        
--    end if;
--      o_fetch <= fetch;
--end process fetching;


end architecture beh;
